const http = require('http'); // Import Node.js core module
const https = require('https'); // Import Node.js core module
const querystring = require('querystring');
var fakePersistantStorage = []; // Doesn't survive a restart

var server = http.createServer(function (req, res) {   //create web server
    let urlWithoutParams = req.url;
    if (urlWithoutParams.indexOf('?') > -1){
        urlWithoutParams = urlWithoutParams.substring(0, urlWithoutParams.indexOf('?'));
    }
    console.log(urlWithoutParams);
    if (urlWithoutParams == '/') { //check the URL of the current request
        // set response header
        res.writeHead(200, { 'Content-Type': 'text/html' }); 
        
        let body = `<html>
        <head>
        <script>
            function goToEndpoint(){
                // First store the provided endpoint, so we validate server side URL poisoning from the callback to fetch the API key.
                localStorage.setItem('eLabNextEndPoint', document.getElementById('endpoint').value);
                location.href = 'https://' + document.getElementById('endpoint').value
                + '/members/fn/externalAuth/?rootVar=' + encodeURIComponent(document.getElementById('rootVar').value)
                + '&state=' + encodeURIComponent(document.getElementById('state').value)
                ;
            }
            
        </script>
        </head>
        <body>
            <h2>Authentication Example</h2>
            <p>This assumes the end-user already has your add-on installed with the rootVar <i>MyExampleRootVar</i> 
            that has a configured
            <i>Authentication Return URL</i> in the global variable <i>MyExampleRootVar.authReturnURL</i></p>
            <p>You may pass an optional <i>'state'</i> parameter that will be echoed back.</p>
            <p>Note that you must redirect the user to the proper endpoint to allow both public cloud and private cloud installations to work with your application.
            <form method="GET" action="" onsubmit="goToEndpoint();return false;">
            <label>Endpoint:</label><input type="text" name="endpoint" id="endpoint" value="local.elabjournal.com" /><br/>
            <button id="btnGetToken">Ask Consent and get API Token from endpoint</button>
            <input type="hidden" name="rootVar" id="rootVar" value="MyExampleRootVar">
            <input type="hidden" name="state" id="state" value="myStateString">
            </form>
        </body>
        </html>`
        // set response content    
        res.write(body);
        res.end();
    
    }
    else if (urlWithoutParams == "/checkAccess") {
        let params= querystring.parse(req.url.substring(req.url.indexOf('?')+1));
        console.log('received', params);
        let hasUserAPIKey = false;
        // Check if we already have an API key for this user. In real applications, you would also store the endpoint along with the userID, as 
        // the userIDs are only unique per endpoint.
        if (fakePersistantStorage.indexOf(parseInt(params.userID)) > -1){
            hasUserAPIKey = true;
        } 

        res.writeHead(200, { 'Content-Type': 'application/json' });
        let body = `{"hasAPIKey":` + hasUserAPIKey + `}`;
        res.write(body);
        res.end();
    }
    else if (urlWithoutParams == "/fetchToken") {

        let whenTokenReceived = function(obj){
            res.writeHead(200, { 'Content-Type': 'text/html' });
                let body = `<html>
                <head>
                <script>
                    if (localStorage.getItem('eLabNextEndPoint') != request('endpoint')){
                           alert('Security risk: Endpoint has been changed. This validation should be done differently in production code, e.g. by first checking the endpoint URL parameter with the localStorage key value before allowing it to be used in your server side token exchange.');
                    }
                    function request(name) {
                        name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
                        var regexS = "[\\?&]" + name + "=([^&#]*)";
                        var regex = new RegExp(regexS);
                        var results = regex.exec(window.location.href);
                        if (results == null)
                            return "";
                        else
                            return unescape(results[1]);
                    }
                </script>
                </head>
                <body>
                    <h2>Authentication Example</h2>
                    <p>The Request Token needs to be used within 60 seconds to fetch a long-living API token.</p>
                    <p>Request Token: ` +  params.requestToken + `</p>
                    <p>(Optional) State: ` +  params.state + `</p>
                    <p>API Token (use these for further API requests on behalf of the user): ` +  obj.apiToken + `</p>
                    <p>Endpoint: ` +  params.endpoint + `</p>
                    <p>UserID: ` +  obj.userID + `</p>
                    <p>Username: ` +  obj.username + `</p>
                </body>
                </html>`
                // set response content    
                res.write(body);
                res.end();
        }

        // Get the requestToken
        let params= querystring.parse(req.url.substring(req.url.indexOf('?')+1))
        let requestToken = params.requestToken;
        console.log('received requestToken',params);

        // Build the post string from an object
        var post_data = querystring.stringify({
            "requestToken": requestToken
          });
      
        try {
            var post_req = http.request({
                host: params.endpoint, // WARNING; never allow just the URL parameter endpoint to be used here. It should first be validated with the user provided input (in this example stored in localStorage.getItem('eLabNextEndPoint'))
                port: '80',
                path: '/api/v1/auth/user/exchangeToken',
                method: 'POST',
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded',
                    'Content-Length': Buffer.byteLength(post_data)
                }
            }, function(res) {
                console.log('getting response', res.statusCode);
                res.setEncoding('utf8'); 
                if (res.statusCode != 200){
                    whenTokenReceived("Error: received StatusCode " + res.statusCode);
                } else {
                    res.on('data', function (data) {
                        var obj = JSON.parse(data);
                        console.log(obj);
                        console.log('Adding',obj.userID,'to fake Persistant Storage. In real applications, you would also store the endpoint along with the userID.');
                        fakePersistantStorage.push(obj.userID);
                        whenTokenReceived(obj);
                    });
                }
            });
        } catch(err){

            res.write(err);
                res.end();
        }

        // post the data
        post_req.write(post_data);
        post_req.end();
    }
    else
        res.end('Invalid Request!');

});

server.listen(5000); //6 - listen for any incoming requests

console.log('Node.js web server at port 5000 is running..');
